/*
Storing JSON in a table, specifying constraints for data integrity
*/
DROP TABLE IF EXISTS #PizzaOrders;

CREATE TABLE #PizzaOrders (
    [ID] INT PRIMARY KEY IDENTITY,
    [Order] NVARCHAR(512)--must use NVARCHAR datatype
);


--On INSERT or UPDATE, this constraint verifies that the JSON document is properly formatted.
ALTER TABLE #PizzaOrders
ADD CONSTRAINT [JSON Format Check] CHECK (ISJSON([Order])=1);


--This constraint verifies that the the COST value is numeric
ALTER TABLE #PizzaOrders
ADD CONSTRAINT [Cost should be number] CHECK (ISNUMERIC(JSON_VALUE([Order], '$.Cost'))>0)


--Try to insert non-JSON value
INSERT INTO #PizzaOrders ([Order])
VALUES (N'Not JSON');


INSERT INTO #PizzaOrders ([Order])
VALUES (N'{"FirstName":"Homer","LastName":"Simpson","Size":"Large","Crust":"Thin","Toppings":["Sausage","Mushrooms","Peppers"],"Cost":17.95}')
		,(N'{"FirstName":"Bilbo","LastName":"Baggins","Size":"Small","Crust":"Thin","Toppings":["Ham","Pineapple"],"Cost":5.95}')
		,(N'{"FirstName":"Bruce","LastName":"Wayne","Size":"X-Large","Crust":"Hand Tossed","Toppings":["Pepperoni","Sausage","Ham","Mushrooms","Onions","Peppers"],"Cost":24.00,"Qty":7}')
		,(N'{"FirstName":"The","LastName":"Doctor","Size":"Large","Crust":"Thin","Toppings":["Extra Cheese"],"Cost":9.99}')
		,(N'{"FirstName":"Homer","LastName":"Simpson","Size":"Large","Crust":"Thick","Toppings":["Ham","Pineapple"],"Cost":7.99}')
		,(N'{"FirstName":"Darth","LastName":"Vader","Size":"Family","Crust":"Stuffed Crust","Toppings":["Ham","Mushrooms","Onions"],"Cost":27.95,"Qty":3}')
		,(N'{"FirstName":"Homer","LastName":"Simpson","Size":"Large","Crust":"Thin","Toppings":["Sausage","Mushrooms","Peppers"],"Cost":11.50}')
		,(N'{"FirstName":"Homer","LastName":"Simpson","Size":"Large","Crust":"Thick","Toppings":["Ham","Pineapple"],"Cost":7.99}')
		,(N'{"FirstName":"Bilbo","LastName":"Baggins","Size":"Small","Crust":"Thin","Toppings":["Ham","Pineapple"],"Cost":15.95}')
		,(N'{"FirstName":"Bruce","LastName":"Wayne","Size":"X-Large","Crust":"Hand Tossed","Toppings":["Pepperoni","Sausage","Ham","Mushrooms","Onions","Peppers"],"Cost":14.55}')
		,(N'{"FirstName":"Homer","LastName":"Simpson","Size":"Large","Crust":"Thin","Toppings":["Sausage","Mushrooms","Peppers"],"Cost":11.50}');


SELECT * FROM #PizzaOrders;

--Parse JSON and return as regular query result
SELECT [FirstName] = JSON_VALUE([Order],'$.FirstName')
	,[LastName] = JSON_VALUE([Order],'$.LastName')
	,[Size] = JSON_VALUE([Order],'$.Size')
	,[Crust] = JSON_VALUE([Order],'$.Crust')
	,[Cost] = JSON_VALUE([Order],'$.Cost')
	,[Order QTY] = JSON_VALUE([Order],'$.Qty')
	,[Toppings] = JSON_QUERY([Order],'$.Toppings')
FROM #PizzaOrders;

--Parse JSON, and use COUNT and GROUP BY
SELECT [FirstName] = JSON_VALUE([Order],'$.FirstName')
	,[LastName] = JSON_VALUE([Order],'$.LastName')
	,[Orders Placed] = COUNT(*)
FROM #PizzaOrders
GROUP BY JSON_VALUE([Order],'$.FirstName'),JSON_VALUE([Order],'$.LastName')
ORDER BY [Orders Placed] DESC;

--Parse JSON, and use COUNT, AVG, SUM, and GROUP BY
SELECT [FirstName] = JSON_VALUE([Order],'$.FirstName')
	,[LastName] = JSON_VALUE([Order],'$.LastName')
	,[Orders Placed] = COUNT(*)
	,[AVG Spend] = AVG (CAST(JSON_VALUE([Order],'$.Cost') AS FLOAT))
	,[Total Spent] = SUM (CAST(JSON_VALUE([Order],'$.Cost') AS FLOAT))
	,[Total QTY Ordered] = SUM (CAST(JSON_VALUE([Order],'$.Qty') AS FLOAT))
FROM #PizzaOrders
GROUP BY JSON_VALUE([Order],'$.FirstName'),JSON_VALUE([Order],'$.LastName')
ORDER BY [Orders Placed] DESC;

--You can even throw in ISNULL!
SELECT [FirstName] = JSON_VALUE([Order],'$.FirstName')
	,[LastName] = JSON_VALUE([Order],'$.LastName')
	,[Orders Placed] = COUNT(*)
	,[AVG Spend] = AVG (CAST(JSON_VALUE([Order],'$.Cost') AS FLOAT))
	,[Total Spent] = SUM (CAST(JSON_VALUE([Order],'$.Cost') AS FLOAT))
	,[Total QTY Ordered] = ISNULL(SUM (CAST(JSON_VALUE([Order],'$.Qty') AS FLOAT)),1)
FROM #PizzaOrders
GROUP BY JSON_VALUE([Order],'$.FirstName'),JSON_VALUE([Order],'$.LastName')
ORDER BY [Orders Placed] DESC;

